# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""add-missing-index-and-foreign-key

Revision ID: 74c97dca93d0
Revises: 47d79ffdc582
Create Date: 2020-03-27 02:51:30.959226

"""

# revision identifiers, used by Alembic.
revision = '74c97dca93d0'
down_revision = '47d79ffdc582'
branch_labels = None
depends_on = None

from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('container', schema=None) as batch_op:
        batch_op.create_foreign_key(
            None, 'container', ['capsule_id'], ['id'], ondelete='CASCADE')
    # ### end Alembic commands ###
